// Analyser_MultiDevice.cpp : Defines the entry point for the console application.
//

#include <conio.h>
#include <direct.h>
#include <windows.h>

#include "stdafx.h"

extern "C" {
   #include "AnalyserC.h"
}

HANDLE hStartJob =  NULL;
HANDLE hInst1ThreadStarted =  NULL;
HANDLE hInst2ThreadStarted =  NULL;

int hInst1;
int hInst2;

// Forward declarations
DWORD WINAPI Dev1Thread(LPVOID lpParam);
DWORD WINAPI Dev2Thread(LPVOID lpParam);

int _tmain(int argc, _TCHAR* argv[])
{
   //
   // Go to the work directory
   //
   printf("\nGoing to the work directory...\n");
   #ifdef _DEBUG
      _chdir("..\\debug");
   #else
      _chdir("..\\release");5
   #endif;

   //
   // Open library
   //
   printf("Opening the AnalyserC library...\n");
   hInst1 = ana_AnalyserC();

   //////////////////////////////////
   // Run a firt job with device 1 //
   //////////////////////////////////

   //
   // Select device 1
   //
   printf("\nSelecting device 1...\n");
   ana_SelectDevice("03010922004");

   //
   // Read configuration and data file
   //
   printf("Reading configuration and data file for device 1...\n");
   ana_ReadConfFileAnalyser("..\\cfg_files\\analyser_simple.cfg", true);

   //
   // Send the data to the device
   //
   printf("Running the job with device 1...\n");
   int NrOfSamples = 100;
   ana_AnalyserRun(NrOfSamples, false, true);

   //
   // Get the captured data
   //
   short Data;
   printf("First 10 samples from device 1:\n");
   for (int i=0; i<10; i++) {
      ana_GetCapturedData(i, &Data);
      printf("\t0x%04x\n", Data);
   }

   //////////////////////////////////
   // Run a firt job with device 2 //
   //////////////////////////////////

   //
   // Select device 2
   //
   printf("\nSelecting device 2...\n");
   ana_SelectDevice("0101082505E");

   //
   // Read configuration and data file
   //
   printf("Reading configuration and data file for device 2...\n");
   ana_ReadConfFileAnalyser("..\\cfg_files\\analyser_simple.cfg", true);

   //
   // Send the data to the device
   //
   printf("Running the job with device 2...\n");
   NrOfSamples = 100;
   ana_AnalyserRun(NrOfSamples, false, true);

   //
   // Get the captured data
   //
   printf("First 10 samples from device 2:\n");
   for (int i=0; i<10; i++) {
      ana_GetCapturedData(i, &Data);
      printf("\t0x%04x\n", Data);
   }

   /////////////////////////////////////////////////////////////
   // Start threads to run jobs simultaneously on two devices //
   /////////////////////////////////////////////////////////////

   hInst2 = ana_CreateInstance(); // open a second instance

   // Link eash instance to a device
   printf("\nSelect instance 1 and device 1...\n");
   ana_SelectInstance(hInst1);
   ana_SelectDevice("0101082505E");
   printf("Reading configuration and data file for device 1...\n");
   ana_ReadConfFileAnalyser("..\\cfg_files\\analyser_simple.cfg", true);

   printf("Select instance 2 and device 2...\n");
   ana_SelectInstance(hInst2);
   ana_SelectDevice("03010922004");
   printf("Reading configuration and data file for device 2...\n");
   ana_ReadConfFileAnalyser("..\\cfg_files\\analyser_simple.cfg", true);

   printf("\nStarting threads for device 1 & 2...\n");
   hStartJob = CreateEvent(NULL, TRUE, FALSE, NULL);

   hInst1ThreadStarted = CreateEvent(NULL, FALSE, FALSE, NULL);
   HANDLE hInst1Thread = CreateThread(NULL, 0, Dev1Thread, NULL, 0, NULL);
   WaitForSingleObject(hInst1ThreadStarted, INFINITE);

   hInst2ThreadStarted = CreateEvent(NULL, FALSE, FALSE, NULL);
   HANDLE hInst2Thread = CreateThread(NULL, 0, Dev2Thread, NULL, 0, NULL);
   WaitForSingleObject(hInst2ThreadStarted, INFINITE);

   printf("Running jobs on device 1 & 2 simultaneously...\n");
   SetEvent(hStartJob);
   WaitForSingleObject(hInst1Thread, INFINITE);
   WaitForSingleObject(hInst2Thread, INFINITE);

   printf("Closing threads...\n");
   CloseHandle(hStartJob);
   CloseHandle(hInst1ThreadStarted);
   CloseHandle(hInst2ThreadStarted);

   //
   // Close the library
   //
   printf("\nClosing device 1...\n");
   ana_SelectInstance(hInst1);
   ana_Terminate();
   printf("Closing device 2...\n");
   ana_SelectInstance(hInst2);
   ana_Terminate();

   //
   // Wait for user input to close the window
   //
   printf("Press any key...\n", 1);
   int ch = _getch();

	return 0;
}

DWORD WINAPI Dev1Thread(LPVOID lpParam)
{
   SetEvent(hInst1ThreadStarted);
   WaitForSingleObject(hStartJob, INFINITE);
   ana_AnalyserRunH(100, false, true, hInst1);
   return 0;
}

DWORD WINAPI Dev2Thread(LPVOID lpParam)
{
   SetEvent(hInst2ThreadStarted);
   WaitForSingleObject(hStartJob, INFINITE);
   ana_AnalyserRunH(100, false, true, hInst2);
   return 0;
}
